use {
    crate::prelude::{CertificateStore, GetFingerprint},
    rustls::{
        client::{ServerCertVerified, ServerCertVerifier},
        Certificate,
    },
    std::{
        borrow::BorrowMut,
        sync::{Arc, Mutex},
        time,
    },
};

#[derive(Debug)]
/// A verifier is used to verify certificates sent by the receiving server
/// during the tls handshake.
pub struct Verifier<S: CertificateStore> {
    /// An item which serves as storage for certificates
    pub store: Arc<Mutex<S>>,
}

impl<S: CertificateStore> ServerCertVerifier for Verifier<S> {
    fn verify_server_cert(
        &self,
        end_entity: &Certificate,
        _intermediates: &[Certificate],
        server_name: &rustls::ServerName,
        _scts: &mut dyn Iterator<Item = &[u8]>,
        _ocsp_response: &[u8],
        _now: time::SystemTime,
    ) -> Result<ServerCertVerified, rustls::Error> {
        let fp = end_entity
            .fingerprint()
            .map_err(|e| rustls::Error::General(e.to_string()))?;
        let name = match server_name {
            rustls::ServerName::DnsName(n) => n.as_ref().to_string(),
            rustls::ServerName::IpAddress(ip) => ip.to_string(),
            _ => todo!(),
        };
        let mut store = self.store.lock().unwrap();
        if let Some(fingerprint) = store.get_certificate(&name) {
            // TODO: needs a lot more checking for certificate validity
            if fingerprint == fp.fingerprint {
                for n in fp.names {
                    if n == name {
                        return Ok(ServerCertVerified::assertion());
                    }
                }
                Err(rustls::Error::InvalidCertificate(
                    rustls::CertificateError::NotValidForName,
                ))
            } else {
                Err(rustls::Error::InvalidCertificate(
                    rustls::CertificateError::NotValidForName,
                ))
            }
        } else {
            if !store.contains_certificate(&name) {
                let _key = store
                    .borrow_mut()
                    .insert_certificate(&name, &fp.fingerprint);
            }
            return Ok(ServerCertVerified::assertion());
        }
    }
}

impl<T: CertificateStore> From<T> for Verifier<T> {
    fn from(value: T) -> Self {
        Self {
            store: Arc::new(Mutex::new(value)),
        }
    }
}

impl<T: CertificateStore> Verifier<T> {
    pub fn new(store: T) -> Self {
        store.into()
    }
}
